@@ -1,18 +1,18 @@
 #!/bin/sh
 
-# Copyright (c) 2012 - 2017 dak180 and contributors. See
+# Copyright (c) 2012 - 2021 dak180 and contributors. See
 # https://opensource.org/licenses/mit-license.php or the included
 # COPYING.md for licence terms.
 #
 # autorevision - extracts metadata about the head version from your
 # repository.
 
-# shellcheck disable=SC2154
+# shellcheck disable=SC2154,SC2236,SC2030,SC2031
 
 # Usage message.
 arUsage() {
 	tee >&2 << EOF
-usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-e name] [-U] [-q] [-V]
+usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-e name] [-U] [-C path] [-q] [-V]
 	Options include:
 	-t output-type		= specify output type
 	-s symbol		= specify symbol output
@@ -20,6 +20,7 @@ usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-e name]
 	-f			= force the use of cache data
 	-e name			= set a different output name for VCS_EXTRA
 	-U			= check for untracked files in svn
+	-C path			= change to path before working on repository
 	-q			= Silence warnings
 	-V			= emit version and exit
 	-?			= help message
@@ -74,7 +75,7 @@ EOF
 
 # Config
 ARVERSION="&&ARVERSION&&"
-while getopts ":t:o:s:e:VfqU" OPTION; do
+while getopts ":t:o:s:e:C:VfqU" OPTION; do
 	case "${OPTION}" in
 		t)
 			AFILETYPE="${OPTARG}"
@@ -91,6 +92,9 @@ while getopts ":t:o:s:e:VfqU" OPTION; do
 		e)
 			EXTRA_NAME="${OPTARG}"
 		;;
+		C)
+			CHANGE_TO_DIR="${OPTARG}"
+		;;
 		U)
 			UNTRACKEDFILES="1"
 		;;
@@ -135,6 +139,26 @@ else
 	LOCAL=""
 fi
 
+#check if needed software is installed
+commands="sed
+cut
+grep
+wc
+tail
+basename
+dirname
+tee
+getopts
+tr
+cmp"
+
+for command in ${commands}; do
+	if ! command -v "${command}" > /dev/null 2>&1; then
+		echo "${command} is missing, please install"
+		exit 100
+	fi
+done
+
 # Make sure that the path we are given is one we can source
 # (dash, we are looking at you).
 if [ ! -z "${CACHEFILE}" ] && ! echo "${CACHEFILE}" | grep -q '^\.*/'; then
@@ -149,13 +173,13 @@ GENERATED_HEADER="Generated by autorevision - do not hand-hack!"
 # For git repos
 # shellcheck disable=SC2039,SC2164,SC2155
 gitRepo() {
-	${LOCAL} oldPath="${PWD}"
+	${LOCAL} oldPath="$(pwd)"
 
 	cd "$(git rev-parse --show-toplevel)"
 
 	VCS_TYPE="git"
 
-	VCS_BASENAME="$(basename "${PWD}")"
+	VCS_BASENAME="$(basename "$(pwd)")"
 
 	${LOCAL} currentRev="$(git rev-parse HEAD)"
 
@@ -194,7 +218,7 @@ gitRepo() {
 	VCS_COMMIT_OPENPGP="$(git verify-commit --raw "${currentRev}" 2>&1 | grep "VALIDSIG" | cut -d ' ' -f 12)"
 
 	# Current branch
-	VCS_BRANCH="$(git rev-parse --symbolic-full-name --verify "$(git name-rev --name-only --no-undefined "${currentRev}" 2>/dev/null)" 2>/dev/null | sed -e 's:refs/heads/::' | sed -e 's:refs/::')"
+	VCS_BRANCH="$(git rev-parse --symbolic-full-name --verify "$(git name-rev --name-only --no-undefined --exclude='refs/tags/*' "${currentRev}" 2>/dev/null)" 2>/dev/null | sed -e 's:refs/heads/::' | sed -e 's:refs/::')"
 
 	# Cache the description
 	${LOCAL} DESCRIPTION="$(git describe --long --tags "${currentRev}" 2>/dev/null)"
@@ -202,16 +226,18 @@ gitRepo() {
 	# Current or last tag ancestor (empty if no tags)
 	VCS_TAG="$(echo "${DESCRIPTION}" | sed -e "s:-g${VCS_SHORT_HASH}\$::" -e 's:-[0-9]*$::')"
 
-	# Determine if the tag is annotated.
-	${LOCAL} tagType="$(git for-each-ref --format='%(objecttype)' "$(git rev-parse --symbolic-full-name --verify "${VCS_TAG}")")"
-	if [ ! "${tagType}" = "tag" ]; then
-		# Let the user know if the current tag is not annotated.
-		if [ ! "${WARNHIDE}" = "1" ]; then
-		echo "note: The most recent tag is not annotated; if it is intended to be a release you may want to fix this." 1>&2
+	if [ ! -z "${VCS_TAG}" ]; then
+		# Determine if the tag is annotated.
+		${LOCAL} tagType="$(git for-each-ref --format='%(objecttype)' "$(git rev-parse --symbolic-full-name --verify "${VCS_TAG}")")"
+		if [ ! "${tagType}" = "tag" ]; then
+			# Let the user know if the current tag is not annotated.
+			if [ ! "${WARNHIDE}" = "1" ]; then
+				echo "note: The most recent tag is not annotated; if it is intended to be a release you may want to fix this." 1>&2
+			fi
+		else
+			# You cannot have a signed tag that is not annotated.
+			VCS_TAG_OPENPGP="$(git verify-tag --raw "${VCS_TAG}" 2>&1 | grep "VALIDSIG" | cut -d ' ' -f 12)"
 		fi
-	else
-		# You cannot have a signed tag that is not annotated.
-		VCS_TAG_OPENPGP="$(git verify-tag --raw "${VCS_TAG}" 2>&1 | grep "VALIDSIG" | cut -d ' ' -f 12)"
 	fi
 
 	# Distance to last tag or an alias of VCS_NUM if there is no tag
@@ -244,13 +270,13 @@ gitRepo() {
 # For hg repos
 # shellcheck disable=SC2039,SC2164
 hgRepo() {
-	${LOCAL} oldPath="${PWD}"
+	${LOCAL} oldPath="$(pwd)"
 
 	cd "$(hg root)"
 
 	VCS_TYPE="hg"
 
-	VCS_BASENAME="$(basename "${PWD}")"
+	VCS_BASENAME="$(basename "$(pwd)")"
 
 	VCS_UUID="$(hg log -r "0" -l 1 --template '{node}\n')"
 
@@ -297,13 +323,13 @@ hgRepo() {
 # For bzr repos
 # shellcheck disable=SC2039,SC2164
 bzrRepo() {
-	${LOCAL} oldPath="${PWD}"
+	${LOCAL} oldPath="$(pwd)"
 
 	cd "$(bzr root)"
 
 	VCS_TYPE="bzr"
 
-	VCS_BASENAME="$(basename "${PWD}")"
+	VCS_BASENAME="$(basename "$(pwd)")"
 
 	# Currently unimplemented because more investigation is needed.
 	VCS_UUID=""
@@ -347,24 +373,24 @@ bzrRepo() {
 # For svn repos
 # shellcheck disable=SC2039,SC2164,SC2155
 svnRepo() {
-	${LOCAL} oldPath="${PWD}"
+	${LOCAL} oldPath="$(pwd)"
 
 	VCS_TYPE="svn"
 
-	case "${PWD}" in
+	case "$(pwd)" in
 	/*trunk*|/*branches*|/*tags*)
-		${LOCAL} fn="${PWD}"
+		${LOCAL} fn="$(pwd)"
 		while [ "$(basename "${fn}")" != 'trunk' ] && [ "$(basename "${fn}")" != 'branches' ] && [ "$(basename "${fn}")" != 'tags' ] && [ "$(basename "${fn}")" != '/' ]; do
 			${LOCAL} fn="$(dirname "${fn}")"
 		done
 		${LOCAL} fn="$(dirname "${fn}")"
 		if [ "${fn}" = '/' ]; then
-			VCS_BASENAME="$(basename "${PWD}")"
+			VCS_BASENAME="$(basename "$(pwd)")"
 		else
 			VCS_BASENAME="$(basename "${fn}")"
 		fi
 		;;
-	*) VCS_BASENAME="$(basename "${PWD}")" ;;
+	*) VCS_BASENAME="$(basename "$(pwd)")" ;;
 	esac
 
 	VCS_UUID="$(svn info --xml | sed -n -e 's:<uuid>::' -e 's:</uuid>::p')"
@@ -402,10 +428,10 @@ svnRepo() {
 	VCS_SHORT_HASH="${VCS_NUM}"
 
 	# Current branch
-	case "${PWD}" in
+	case "$(pwd)" in
 	/*trunk*|/*branches*|/*tags*)
 		${LOCAL} lastbase=""
-		${LOCAL} fn="${PWD}"
+		${LOCAL} fn="$(pwd)"
 		while :
 		do
 			base="$(basename "${fn}")"
@@ -1230,10 +1256,12 @@ EOF
 pathSegment() {
 	${LOCAL} pathz="${1}"
 	${LOCAL} depth="0"
+	${LOCAL} last_pathz=""
 
 	if [ ! -z "${pathz}" ]; then
 		# Continue until we are at / or there are no path separators left.
-		while [ ! "${pathz}" = "/" ] && [ ! "${pathz}" = "$(echo "${pathz}" | sed -e 's:/::')" ]; do
+		while [ ! "${pathz}" = "/" ] && [ ! "${pathz}" = "${last_pathz}" ]; do
+			last_pathz="${pathz}"
 			pathz="$(dirname "${pathz}")"
 			depth="$((depth+1))"
 		done
@@ -1333,8 +1361,20 @@ if [ -f "${CACHEFILE}" ] && [ "${CACHEFORCE}" = "1" ]; then
 else
 	# If a value is not set through the environment set VCS_EXTRA to nothing.
 	: "${VCS_EXTRA:=""}"
+
+	if [ ! -z "${CHANGE_TO_DIR}" ]; then
+		originalPath="$(pwd)"
+		# shellcheck disable=SC2164
+		cd "${CHANGE_TO_DIR}"
+	fi
+
 	repoTest
 
+	if [ ! -z "${originalPath}" ]; then
+		# shellcheck disable=SC2164
+		cd "${originalPath}"
+	fi
+
 	if [ -f "${CACHEFILE}" ] && [ "${REPONUM}" = "0" ]; then
 		# We are not in a repo; try to use a previously generated cache to populate our symbols.
 		. "${CACHEFILE}"
@@ -1368,89 +1408,129 @@ fi
 
 # -s output is handled here.
 if [ ! -z "${VAROUT}" ]; then
-	if [ "${VAROUT}" = "VCS_TYPE" ]; then
-		echo "${VCS_TYPE}"
-	elif [ "${VAROUT}" = "VCS_BASENAME" ]; then
-		echo "${VCS_BASENAME}"
-	elif [ "${VAROUT}" = "VCS_NUM" ]; then
-		echo "${VCS_NUM}"
-	elif [ "${VAROUT}" = "VCS_DATE" ]; then
-		echo "${VCS_DATE}"
-	elif [ "${VAROUT}" = "VCS_BRANCH" ]; then
-		echo "${VCS_BRANCH}"
-	elif [ "${VAROUT}" = "VCS_TAG" ]; then
-		echo "${VCS_TAG}"
-	elif [ "${VAROUT}" = "VCS_TICK" ]; then
-		echo "${VCS_TICK}"
-	elif [ "${VAROUT}" = "VCS_FULL_HASH" ]; then
-		echo "${VCS_FULL_HASH}"
-	elif [ "${VAROUT}" = "VCS_SHORT_HASH" ]; then
-		echo "${VCS_SHORT_HASH}"
-	elif [ "${VAROUT}" = "VCS_WC_MODIFIED" ]; then
-		echo "${VCS_WC_MODIFIED}"
-	elif [ "${VAROUT}" = "VCS_ACTION_STAMP" ]; then
-		echo "${VCS_ACTION_STAMP}"
-	else
-		echo "error: Not a valid output symbol." 1>&2
-		exit 1
-	fi
+	case "${VAROUT}" in
+		VCS_TYPE)
+			echo "${VCS_TYPE}"
+		;;
+		VCS_BASENAME)
+			echo "${VCS_BASENAME}"
+		;;
+		VCS_NUM)
+			echo "${VCS_NUM}"
+		;;
+		VCS_DATE)
+			echo "${VCS_DATE}"
+		;;
+		VCS_BRANCH)
+			echo "${VCS_BRANCH}"
+		;;
+		VCS_TAG)
+			echo "${VCS_TAG}"
+		;;
+		VCS_TICK)
+			echo "${VCS_TICK}"
+		;;
+		VCS_FULL_HASH)
+			echo "${VCS_FULL_HASH}"
+		;;
+		VCS_SHORT_HASH)
+			echo "${VCS_SHORT_HASH}"
+		;;
+		VCS_WC_MODIFIED)
+			echo "${VCS_WC_MODIFIED}"
+		;;
+		VCS_ACTION_STAMP)
+			echo "${VCS_ACTION_STAMP}"
+		;;
+		*)
+			echo "error: ${VAROUT} is not a valid output symbol." 1>&2
+			exit 1
+		;;
+	esac
 fi
 
 
 # Detect requested output type and use it.
 if [ ! -z "${AFILETYPE}" ]; then
-	if [ "${AFILETYPE}" = "c" ]; then
-		cOutput
-	elif [ "${AFILETYPE}" = "h" ]; then
-		hOutput
-	elif [ "${AFILETYPE}" = "xcode" ]; then
-		xcodeOutput
-	elif [ "${AFILETYPE}" = "swift" ]; then
-		swiftOutput
-	elif [ "${AFILETYPE}" = "sed" ]; then
-		sedOutput
-	elif [ "${AFILETYPE}" = "sh" ]; then
-		shOutput
-	elif [ "${AFILETYPE}" = "py" ] || [ "${AFILETYPE}" = "python" ]; then
-		pyOutput
-	elif [ "${AFILETYPE}" = "pl" ] || [ "${AFILETYPE}" = "perl" ]; then
-		plOutput
-	elif [ "${AFILETYPE}" = "lua" ]; then
-		luaOutput
-	elif [ "${AFILETYPE}" = "php" ]; then
-		phpOutput
-	elif [ "${AFILETYPE}" = "ini" ]; then
-		iniOutput
-	elif [ "${AFILETYPE}" = "js" ]; then
-		jsOutput
-	elif [ "${AFILETYPE}" = "json" ]; then
-		jsonOutput
-	elif [ "${AFILETYPE}" = "java" ]; then
-		javaOutput
-	elif [ "${AFILETYPE}" = "javaprop" ]; then
-		javapropOutput
-	elif [ "${AFILETYPE}" = "csharp" ]; then
-		csharpOutput
-	elif [ "${AFILETYPE}" = "tex" ]; then
-		texOutput
-	elif [ "${AFILETYPE}" = "m4" ]; then
-		m4Output
-	elif [ "${AFILETYPE}" = "scheme" ]; then
-		schemeOutput
-	elif [ "${AFILETYPE}" = "clojure" ]; then
-		clojureOutput
-	elif [ "${AFILETYPE}" = "rpm" ]; then
-		rpmOutput
-	elif [ "${AFILETYPE}" = "hpp" ]; then
-		hppOutput
-	elif [ "${AFILETYPE}" = "matlab" ]; then
-		matlabOutput
-	elif [ "${AFILETYPE}" = "octave" ]; then
-		octaveOutput
-	elif [ "${AFILETYPE}" = "cmake" ]; then
-		cmakeOutput
-	else
-		echo "error: Not a valid output type." 1>&2
-		exit 1
-	fi
+	case "${AFILETYPE}" in
+		c)
+			cOutput
+		;;
+		h)
+			hOutput
+		;;
+		xcode)
+			xcodeOutput
+		;;
+		swift)
+			swiftOutput
+		;;
+		sed)
+			sedOutput
+		;;
+		sh)
+			shOutput
+		;;
+		py|python)
+			pyOutput
+		;;
+		pl|perl)
+			plOutput
+		;;
+		lua)
+			luaOutput
+		;;
+		php)
+			phpOutput
+		;;
+		ini)
+			iniOutput
+		;;
+		js)
+			jsOutput
+		;;
+		json)
+			jsonOutput
+		;;
+		java)
+			javaOutput
+		;;
+		javaprop)
+			javapropOutput
+		;;
+		csharp)
+			csharpOutput
+		;;
+		tex)
+			texOutput
+		;;
+		m4)
+			m4Output
+		;;
+		scheme)
+			schemeOutput
+		;;
+		clojure)
+			clojureOutput
+		;;
+		rpm)
+			rpmOutput
+		;;
+		hpp)
+			hppOutput
+		;;
+		matlab)
+			matlabOutput
+		;;
+		octave)
+			octaveOutput
+		;;
+		cmake)
+			cmakeOutput
+		;;
+		*)
+			echo "error: ${AFILETYPE} is not a valid output type." 1>&2
+			exit 1
+		;;
+	esac
 fi
