@@ -9,11 +9,13 @@ extern const int VCS_NUM;
 extern const char *VCS_DATE;
 extern const char *VCS_BRANCH;
 extern const char *VCS_TAG;
+extern const char *VCS_TAG_OPENPGP;
 extern const int VCS_TICK;
 extern const char *VCS_EXTRA;
 
 extern const char *VCS_ACTION_STAMP;
 extern const char *VCS_FULL_HASH;
+extern const char *VCS_COMMIT_OPENPGP;
 extern const char *VCS_SHORT_HASH;
 
 extern const int VCS_WC_MODIFIED;
